/*
 * parser classes for MySpell
 *
 * implemented: text, HTML, TeX, first word
 *
 * Copyright (C) 2002, Laszlo Nemeth
 * License: GNU GPL 2 or later
 *
 */

#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <ctype.h>

#include "textparser.hxx"
#include "firstparser.hxx"
#include "atypes.hxx"

#ifndef WINDOWS
using namespace std;
#endif

extern char * mystrdup(const char * s);

FirstParser::FirstParser(const char * wordchars)
{
	init(wordchars);
}

FirstParser::~FirstParser() 
{
}

char * FirstParser::next_token()
{
	char * latin1;

	for (;;) {
		switch (state)
		{
		case 0: // non word chars
			if (is_wordchar(line[actual][head])) {
				state=1;
				token=head;
			} else if ((latin1 = get_latin1(line[actual] + head))) {
				state = 1;
				token = head;
				head += strlen(latin1);
			}
			break;
		case 1: // wordchar
			if ((latin1 = get_latin1(line[actual] + head))) {
				head += strlen(latin1);
			} else if (! is_wordchar(line[actual][head])) {
				state = 2;
				char * t = (char *) malloc(head - token + 1);
				t[head - token] = '\0';
				if (t) return strncpy(t, line[actual] + token, head - token); 
				fprintf(stderr,"Error - Insufficient Memory\n");
			}
			break;
		case 2: state = 0;
			return NULL;
		}

		if (line[actual][head] == '\0') return NULL;
		head++;
	}
}
